"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const FirebaseRedundancyService_1 = require("./FirebaseRedundancyService");
const NodeSchedule = require("node-schedule");
const Errors_1 = require("../utils/Errors");
class Scheduler {
    constructor(reference, jobCompletionHandler) {
        this.redundancyService = new FirebaseRedundancyService_1.FirebaseRedundancyService(reference);
        this.jobCompletionHandler = jobCompletionHandler;
        this.jobList = new Map();
        this.redundancyService.getAll().then((tasks) => {
            this.queueExisting(tasks);
        }).catch((error) => {
            throw error;
        });
    }
    getJobCallback(key) {
        return () => __awaiter(this, void 0, void 0, function* () {
            if (!this.jobCompletionHandler) {
                return;
            }
            let task = yield this.redundancyService.fetch(key);
            if (!task) {
                return;
            }
            this.jobCompletionHandler(key, task);
            yield this.redundancyService.remove(key);
        });
    }
    scheduleJob(job) {
        if (this.jobList.has(job.getKey())) {
            return;
        }
        const scheduledJob = NodeSchedule.scheduleJob(job.getKey(), job.getScheduledDateTime(), this.getJobCallback(job.getKey()));
        this.jobList.set(job.getKey(), scheduledJob);
    }
    queueExisting(tasks) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!tasks) {
                return;
            }
            tasks.forEach((task) => {
                if (task.getScheduledDateTime().getTime() < Date.now()) {
                    this.jobCompletionHandler(task.getKey(), task);
                    this.redundancyService.remove(task.getKey());
                }
                else {
                    this.scheduleJob(task);
                }
            });
        });
    }
    get(key) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.redundancyService.fetch(key);
        });
    }
    getPendingCount() {
        return this.jobList.size;
    }
    schedule(task) {
        return __awaiter(this, void 0, void 0, function* () {
            if (task.getScheduledDateTime().getTime() < Date.now()) {
                throw new Error(Errors_1.Errors.SCHEDULED_IN_PAST);
            }
            let shouldCreateLocallyOnly = false;
            let existingTask = yield this.redundancyService.fetch(task.getKey());
            if (existingTask) {
                if (this.jobList.has(task.getKey())) {
                    return existingTask;
                }
                shouldCreateLocallyOnly = true;
            }
            this.scheduleJob(task);
            if (!shouldCreateLocallyOnly) {
                yield this.redundancyService.commit(task);
            }
            return task;
        });
    }
    cancel(key) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.jobList.has(key)) {
                return;
            }
            yield this.redundancyService.remove(key);
            let job = this.jobList.get(key);
            if (job)
                job.cancel();
            this.jobList.delete(key);
        });
    }
}
exports.Scheduler = Scheduler;
