"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Errors_1 = require("../utils/Errors");
class Job {
    constructor(key, type, datetime) {
        this.key = null;
        this.type = null;
        this.scheduledDateTime = null;
        this.payload = null;
        if (!key || key.length == 0) {
            throw new Error(Errors_1.Errors.INVALID_JOB_KEY);
        }
        if (!type || type.length == 0) {
            throw new Error(Errors_1.Errors.INVALID_JOB_TYPE);
        }
        this.key = key;
        this.type = type;
        this.scheduledDateTime = datetime;
    }
    static fromJson(data) {
        let job = new Job(data['id'], data['type'], new Date(data['scheduled_date_time']));
        if (data['payload']) {
            job.setPayload(data['payload']);
        }
        return job;
    }
    getKey() {
        return this.key;
    }
    setKey(key) {
        this.key = key;
    }
    getType() {
        return this.type;
    }
    setType(type) {
        this.type = type;
    }
    getScheduledDateTime() {
        return this.scheduledDateTime;
    }
    setScheduledDateTime(date) {
        this.scheduledDateTime = date;
    }
    getPayload() {
        return this.payload;
    }
    setPayload(payload) {
        this.payload = payload;
    }
    asJson() {
        let obj = {
            'id': this.key,
            'type': this.type,
            'scheduled_date_time': this.scheduledDateTime.getTime()
        };
        if (this.payload) {
            obj['payload'] = JSON.stringify(this.payload);
        }
        return obj;
    }
}
exports.Job = Job;
